proc bgerror {error} {
	global mp3tool errorInfo errorCode
	set mp3tool(error) $error
	set mp3tool(errorInfo) $errorInfo
	set mp3tool(errorCode) $errorCode
	if $mp3tool(debugger) {
		if [winfo exists .error] {destroy .error}
		toplevel .error
                wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
                ErrorIcon .error
		label .error.l -text [Trans an_error_has_occured]
                label .error.ll -text [Trans would_you_like_to_send_a_bug_report]
		frame .error.but
                button .error.but.b -text [Trans yes] -command {BugReport .error .error.but.b}
                button .error.but.c -text [Trans no] -command {destroy .error}
                button .error.but.d -text [Trans check_for_new_version] -command {CheckForVersion .error}
		frame .error.text -relief sunken -borderwidth 2
		text .error.text.text -height 10 -width 40
		button .error.text.quit -text [Trans dismiss] -command {destroy .error}
                pack .error.l
                pack .error.ll
		pack .error.but -fill both -expand y
                pack .error.but.b .error.but.c .error.but.d -side left -fill both -expand y
	}
}

proc ErrorIcon {win} {
	global tcl_platform mp3tool
	set platform [array get tcl_platform platform]
	if {$platform == "platform unix"} {
		wm iconbitmap $win @$mp3tool(dir)/images/stop.xbm
	}
}	

proc nick_fix {who} {
	set new_nick ""
	for {set c 0} {$c < [string length $who]} {incr c} {
		switch -exact -- [string range $who $c $c] {
			{[} {
				append new_nick "\\"
				append new_nick "\["
			}
			{]} {
				append new_nick "\\"
				append new_nick "\]"
			}
			\} {
				append new_nick "\\"
				append new_nick "\}"
			}
			\{ {
				append new_nick "\\"
				append new_nick "\{"
			}
			default {
				append new_nick [string range $who $c $c]
			}	
		}
	}
	return $new_nick
}

proc ErrorNotify {routine error} {
	global mp3tool
	if {$mp3tool(debug) != 1} {return}

	if [winfo exists .error_notify] {
		.error_notify.t insert end "Error in $routine\n\n"
		.error_notify.t insert end $error
		.error_notify.t insert end "\n"
		.error_notify.t see end
	} else {
		toplevel .error_notify
		wm title .error_notify "MP3Tool Error Notification"
		ErrorIcon .error_notify
		label .error_notify.la -text "An Error Has occured!"
		label .error_notify.lb -text "Please notify cruise@drunkenbastards.org with the following info"
		text .error_notify.t -height 20 -width 40
		pack .error_notify.la .error_notify.lb -side top -fill y -expand y
		pack .error_notify.t -side bottom -fill both -expand y
		.error_notify.t insert end "Error in $routine\n\n"
		.error_notify.t insert end $error
		.error_notify.t insert end "\n"
		.error_notify.t see end
	}
}


proc SelectOwn {what} {
	set which [HasServerFile [string range $what [expr [string first " " $what] +1] end]]
	if [winfo exists .select] {destroy .select}
	toplevel .select
	wm title .select [Trans offer_window_selection]
	button .select.a -text [Trans play_this] -command "destroy .select;PlaySelected $which" -activeforeground red
 	button .select.b -text [Trans offer_this] -command "destroy .select;SelectedFile \"[string range $what [expr [string first " " $what] +1] end]\"" -activeforeground red
 	button .select.c -text [Trans delete_this] -command "CopyFile $which" -activeforeground red
	pack .select.a -fill both -expand y
	pack .select.b -fill both -expand y
	pack .select.c -fill both -expand y
	wm resizable .select 0 0
}	


proc CopyFile {what} {
	set f [HasServerFile $what]
	file copy $what "/MP3/FIRE/burnme"
	destroy .select
}


proc Trans {what} {
	global INTwords
	if [info exists INTwords($what)] {
		return $INTwords($what)
	} {
		return $what
	}
}

proc NextCommand {win} {
	global mp3tool

	if {[llength $mp3tool(commandlist)] == 0} {return}
	if {[$win.what get] != ""} {
		if {[set idx [lsearch -exact $mp3tool(commandlist) [$win.what get]]] != -1} {
			$win.what delete 0 end
			if {[expr $idx + 1] >= [llength $mp3tool(commandlist)]} {
				set idx -1
			}
			$win.what insert end [lindex $mp3tool(commandlist) [expr $idx +1]]
		} {
			$win.what delete 0 end
			$win.what insert end [lindex $mp3tool(commandlist) 0]
		}
	} {
		$win.what delete 0 end
		$win.what insert end [lindex $mp3tool(commandlist) 0]
	}
}

proc PrevCommand {win} {
	global mp3tool

	if {[llength $mp3tool(commandlist)] == 0} {return}
	if {[$win.what get] != ""} {
		if {[set idx [lsearch -exact $mp3tool(commandlist) [$win.what get]]] != -1} {
			$win.what delete 0 end
			if {[expr $idx - 1] < 0} {
				set idx [llength $mp3tool(commandlist)]
			}
			$win.what insert end [lindex $mp3tool(commandlist) [expr $idx -1]]
		} {
			$win.what delete 0 end
			$win.what insert end [lindex $mp3tool(commandlist) [expr [llength $mp3tool(commandlist)] -1]]
		}
	} {
		$win.what delete 0 end
		$win.what insert end [lindex $mp3tool(commandlist) [expr [llength $mp3tool(commandlist)] -1]]
	}
}
		
proc MP3Props {file} {
	set rate "N/A Kbps"
	set frequency "NA Khz"
	set mod "NA"
	if ![file exists $file] {return "N/A Kbps N/A Khz N/A"}
	if ![file isfile $file] {return "N/A Kbps N/A Khz N/A"}
	set infile [open $file "r"]
	fconfigure $infile -translation binary
	# Just incase there was a hardware error on the read (I've had
	# this trouble)
	set byte(1) 0
	set byte(2) 0
	set byte(3) 0
	set byte(4) 0
	set bits [read $infile 4]
	close $infile
	binary scan $bits c* var1
	set d 0
	foreach v $var1 {
		set byte($d) [expr ($v + 0x100 ) % 0x100]
		incr d
	}
	switch [expr $byte(1) >> 2 & 0x3] {
		2 {set ver 0}
		3 {set ver 1}
		0 {set ver 2}
		default {return "N/A Kbps N/A KHz N/A"}
	}
	set layer [expr 4 - (($byte(1) >> 1) & 0x3)]
	set bitrate [expr ($byte(2) >> 4) & 0x0F]
	set freq [expr ($byte(2) >> 2) & 0x03]
	set mode [expr ($byte(3) >> 6) & 0x03]
	switch $bitrate {
		0 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "N/A Kbps"}
						2 {set rate "N/A Kbps"}
						3 {set rate "N/A Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "N/A Kbps"}
						2 {set rate "N/A Kbps"}
						3 {set rate "N/A Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "N/A Kbps"}
						2 {set rate "N/A Kbps"}
						3 {set rate "N/A Kbps"}
					}
				}
			}
		}
		1 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "32 Kbps"}
						2 {set rate "32 Kbps"}
						3 {set rate "32 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "32 Kbps"}
						2 {set rate "8 Kbps"}
						3 {set rate "8 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "32 Kbps"}
						2 {set rate "8 Kbps"}
						3 {set rate "8 Kbps"}
					}
				}
			}
		}
		2 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "64 Kbps"}
						2 {set rate "48 Kbps"}
						3 {set rate "40 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "48 Kbps"}
						2 {set rate "16 Kbps"}
						3 {set rate "16 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "48 Kbps"}
						2 {set rate "16 Kbps"}
						3 {set rate "16 Kbps"}
					}
				}
			}
		}
		3 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "96 Kbps"}
						2 {set rate "56 Kbps"}
						3 {set rate "48 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "56 Kbps"}
						2 {set rate "24 Kbps"}
						3 {set rate "24 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "56 Kbps"}
						2 {set rate "24 Kbps"}
						3 {set rate "24 Kbps"}
					}
				}
			}
		}
		4 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "128 Kbps"}
						2 {set rate "64 Kbps"}
						3 {set rate "56 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "64 Kbps"}
						2 {set rate "32 Kbps"}
						3 {set rate "32 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "64 Kbps"}
						2 {set rate "32 Kbps"}
						3 {set rate "32 Kbps"}
					}
				}
			}
		}
		5 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "160 Kbps"}
						2 {set rate "80 Kbps"}
						3 {set rate "64 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "80 Kbps"}
						2 {set rate "40 Kbps"}
						3 {set rate "40 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "80 Kbps"}
						2 {set rate "40 Kbps"}
						3 {set rate "40 Kbps"}
					}
				}
			}
		}
		6 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "192 Kbps"}
						2 {set rate "96 Kbps"}
						3 {set rate "80 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "96 Kbps"}
						2 {set rate "48 Kbps"}
						3 {set rate "48 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "96 Kbps"}
						2 {set rate "48 Kbps"}
						3 {set rate "48 Kbps"}
					}
				}
			}
		}
		7 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "224 Kbps"}
						2 {set rate "112 Kbps"}
						3 {set rate "96 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "112 Kbps"}
						2 {set rate "56 Kbps"}
						3 {set rate "56 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "112 Kbps"}
						2 {set rate "56 Kbps"}
						3 {set rate "56 Kbps"}
					}
				}
			}
		}
		8 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "256 Kbps"}
						2 {set rate "128 Kbps"}
						3 {set rate "112 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "128 Kbps"}
						2 {set rate "64 Kbps"}
						3 {set rate "64 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "128 Kbps"}
						2 {set rate "64 Kbps"}
						3 {set rate "64 Kbps"}
					}
				}
			}
		}
		9 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "288 Kbps"}
						2 {set rate "160 Kbps"}
						3 {set rate "128 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "144 Kbps"}
						2 {set rate "80 Kbps"}
						3 {set rate "80 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "144 Kbps"}
						2 {set rate "80 Kbps"}
						3 {set rate "80 Kbps"}
					}
				}
			}
		}
		10 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "320 Kbps"}
						2 {set rate "192 Kbps"}
						3 {set rate "160 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "160 Kbps"}
						2 {set rate "96 Kbps"}
						3 {set rate "96 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "160 Kbps"}
						2 {set rate "96 Kbps"}
						3 {set rate "96 Kbps"}
					}
				}
			}
		}
		11 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "352 Kbps"}
						2 {set rate "224 Kbps"}
						3 {set rate "192 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "176 Kbps"}
						2 {set rate "112 Kbps"}
						3 {set rate "112 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "176 Kbps"}
						2 {set rate "112 Kbps"}
						3 {set rate "112 Kbps"}
					}
				}
			}
		}
		12 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "384 Kbps"}
						2 {set rate "256 Kbps"}
						3 {set rate "224 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "192 Kbps"}
						2 {set rate "128 Kbps"}
						3 {set rate "128 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "192 Kbps"}
						2 {set rate "128 Kbps"}
						3 {set rate "128 Kbps"}
					}
				}
			}
		}
		13 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "416 Kbps"}
						2 {set rate "320 Kbps"}
						3 {set rate "256 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "224 Kbps"}
						2 {set rate "144 Kbps"}
						3 {set rate "144 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "224 Kbps"}
						2 {set rate "144 Kbps"}
						3 {set rate "144 Kbps"}
					}
				}
			}
		}
		14 {
			switch $ver {
				0 {
					switch $layer {
						1 {set rate "448 Kbps"}
						2 {set rate "384 Kbps"}
						3 {set rate "320 Kbps"}
					}
				}
				1 {
					switch $layer {
						1 {set rate "256 Kbps"}
						2 {set rate "160 Kbps"}
						3 {set rate "160 Kbps"}
					}
				}
				2 {
					switch $layer {
						1 {set rate "256 Kbps"}
						2 {set rate "160 Kbps"}
						3 {set rate "160 Kbps"}
					}
				}
			}
		}
	}
	switch $freq {
		0 {
			switch $ver {
				0 {set frequency "44.1 khz"}
				1 {set frequency "22 khz"}
				2 {set frequency "11 khz"}
			}
		}	
		1 {
			switch $ver {
				0 {set frequency "48 khz"}
				1 {set frequency "24 khz"}
				2 {set frequency "8 khz"}
			}
		}	
		2 {
			switch $ver {
				0 {set frequency "32 khz"}
				1 {set frequency "16 khz"}
				2 {set frequency "8 khz"}
			}
		}	
		3 {
			switch $ver {
				0 {set frequency "N/A khz"}
				1 {set frequency "N/A khz"}
				2 {set frequency "N/A khz"}
			}
		}	
	}
	switch $mode {
		0 {set mod "Stereo"}
		1 {set mod "Joint-Stereo"}
		2 {set mod "Dual Channel"}
		3 {set mod "Single Channel (mono)"}
		4 {set mod "Multi channel (mono)"}
	}
	return "$rate $frequency $mod"
}

proc StripColor {what} {
	if {[string first \003 $what] > 0} {
		return [string range $what 0 [expr [string first \003 $what] -1]]
	} else {return $what}
}

proc MP3ToolHelp {} {
	global tcl_platform mp3tool

	if ![winfo exists .help] {
		toplevel .help
		wm title .help "MP3 Tool by Cruise - Help"
		set platform [array get tcl_platform platform]
		if {$platform == "platform unix"} {
			wm iconbitmap .help @$mp3tool(dir)/images/help.xbm
		}
		label .help.info1 -text "MP3Tool [Trans version] $mp3tool(version)"
		label .help.info2 -text "[Trans written_by] Cruise <cruise@drunkenbastards.org>"
		label .help.sep -bg "light blue" \
			-relief raised -text [Trans icon_descriptions]
		frame .help.stop -borderwidth 2 -relief sunken
		frame .help.dump -borderwidth 2 -relief sunken
		frame .help.configure -borderwidth 2 -relief sunken
		frame .help.help -borderwidth 2 -relief sunken
		frame .help.server -borderwidth 2 -relief sunken
		frame .help.save -borderwidth 2 -relief sunken
		frame .help.pinfo -borderwidth 2 -relief sunken
		frame .help.send -borderwidth 2 -relief sunken
		frame .help.recycle -borderwidth 2 -relief sunken
		frame .help.wish -borderwidth 2 -relief sunken
		frame .help.scroll -borderwidth 2 -relief sunken
		frame .help.accept -borderwidth 2 -relief sunken
		frame .help.mouth -borderwidth 2 -relief sunken
		label .help.stop.img -image shutdown
		label .help.stop.txt -text [Trans shuts_down_the_mp3tool_plugin]
		label .help.dump.img -image dump
		label .help.dump.txt -text [Trans remove_a_user_from_your_que]
		label .help.configure.img -image configure
		label .help.configure.txt -text [Trans mp3tool_configurator]
		label .help.help.img -image help
		label .help.help.txt -text [Trans this_help_menu]
		label .help.server.img -image server
		label .help.server.txt -text [Trans load_the_mp3_server_window]
		label .help.save.img -image floppy
		label .help.save.txt -text [Trans saves_your_que_to_disk]
		label .help.pinfo.img -image pinfo
		label .help.pinfo.txt -text [Trans slots_info_window]
		label .help.send.img -image send
		label .help.send.txt -text [Trans send_text_to_the_channel]
		label .help.recycle.img -image recycle
		label .help.recycle.txt -text [Trans view_re-submit_recent_requests]
		label .help.wish.img -image wish
		label .help.wish.txt -text [Trans show_wish_list]
		label .help.scroll.img -image locked
		label .help.scroll.txt -text [Trans lock_offer_window_scrolling]
		label .help.accept.img -image yesnew
		label .help.accept.txt -text [Trans start_stop_offering_files]
		label .help.mouth.img -image mouth
 		label .help.mouth.txt -text [Trans send_a_message_to_everyone_in_your_que]
		pack .help.info1 -expand y -fill both
		pack .help.info2 -expand y -fill both
		pack .help.sep -expand y -fill both
		pack .help.stop -expand y -fill both
		pack .help.configure -expand y -fill both
		pack .help.help -expand y -fill both
		pack .help.server -expand y -fill both
		pack .help.save -expand y -fill both
		pack .help.pinfo -expand y -fill both
		pack .help.send -expand y -fill both
		pack .help.recycle -expand y -fill both
		pack .help.wish -expand y -fill both
		pack .help.scroll -expand y -fill both
		pack .help.accept -expand y -fill both
		pack .help.dump -expand y -fill both
		pack .help.mouth -expand y -fill both
		pack .help.stop.img -side left
		pack .help.stop.txt -side right -fill x -expand y
		pack .help.configure.img -side left
		pack .help.configure.txt -side right -fill x -expand y
		pack .help.help.img -side left
		pack .help.help.txt -side right -fill x -expand y
		pack .help.server.img -side left
		pack .help.server.txt -side right -fill x -expand y
		pack .help.save.img -side left
		pack .help.save.txt -side right -fill x -expand y
		pack .help.pinfo.img -side left
		pack .help.pinfo.txt -side right -fill x -expand y
		pack .help.send.img -side left
		pack .help.send.txt -side right -fill x -expand y
		pack .help.recycle.img -side left
		pack .help.recycle.txt -side right -fill x -expand y
		pack .help.wish.img -side left
		pack .help.wish.txt -side right -fill x -expand y
		pack .help.scroll.img -side left
		pack .help.scroll.txt -side right -fill x -expand y
		pack .help.accept.img -side left
		pack .help.accept.txt -side right -fill x -expand y
		pack .help.dump.img -side left
		pack .help.dump.txt -side right -fill x -expand y
		pack .help.mouth.img -side left
		pack .help.mouth.txt -side right -fill x -expand y
	}
}	

proc StopMP3Tool { } {
	global mp3tool

	if {$mp3tool(client) != "Zircon" && $mp3tool(client) != "OpenVerse"} {
		exit
	} else {
		if {$mp3tool(client) == "Zircon"} {
			global currentNet serving quecount queuser totfin
			upvar #0 OnCode$currentNet OnCode
			set OnCode(CTCP) ""
			set OnCode(PUBLIC) ""
			set OnCode(MSG) ""
		}
		set serving 0
		set quecount 0
		set queuser(0) "Empty List"
		if {[winfo exists .mp3]} {destroy .mp3}
		if {[winfo exists .send]} {destroy .send}
		if {[winfo exists .recycle]} {destroy .recycle}
		if {[winfo exists .help]} {destroy .help}
		if {[winfo exists .server]} {destroy .server}
		if {[winfo exists .configure]} {destroy .configure}
		if {[winfo exists .queuser]} {destroy .queuser}
		if {[winfo exists .select]} {destroy .select}
		if {[winfo exists .file]} {destroy .file}
	}
}

proc SelectMP3 { channel filestring } {
	global mp3tool connections
	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3 Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if [winfo exists .select] {destroy .select}
	toplevel .select
	wm title .select [Trans offer_window_selection]
	button .select.a -text [Trans request_file] -command {REALSelectMP3 $mp3tool(chan) [selection get]} -activeforeground red
 	button .select.b -text [Trans request_list] -command {RequestList [selection get]} -activeforeground red
 	button .select.c -text [Trans request_que] -command {RequestQue [selection get]} -activeforeground red
 	button .select.e -text [Trans request_status] -command {RequestQueStatus [selection get]} -activeforeground red
	button .select.d -text [Trans request_remove] -command {RequestRemove [selection get]} -activeforeground red
	button .select.f -text [Trans ignore_user] -command {RequestIgnore [selection get]} -activeforeground red
	button .select.g -text [Trans add_to_wish_list] -command {WISHSelectMP3 [selection get]} -activeforeground red
	pack .select.a -fill both -expand y
	pack .select.b -fill both -expand y
	pack .select.c -fill both -expand y
	pack .select.e -fill both -expand y
	pack .select.d -fill both -expand y
	pack .select.f -fill both -expand y
	pack .select.g -fill both -expand y
	wm resizable .select 0 0
}

proc InfoButton {what} {
	global mp3tool slotname
	toplevel .select
	wm title .select [Trans slots_info_selection]
 	button .select.b -text [Trans request_list] -command "RequestList \"!$slotname($what)  \"" -activeforeground red
 	button .select.c -text [Trans request_que] -command "RequestQue \"!$slotname($what)  \"" -activeforeground red
 	button .select.e -text [Trans request_status] -command "RequestQueStatus \"!$slotname($what)  \"" -activeforeground red
	button .select.d -text [Trans request_remove] -command "RequestRemove \"!$slotname($what)  \"" -activeforeground red
	button .select.f -text [Trans ignore_user] -command "RequestIgnore \"!$slotname($what)  \"" -activeforeground red
	pack .select.b -fill both -expand y
	pack .select.c -fill both -expand y
	pack .select.e -fill both -expand y
	pack .select.d -fill both -expand y
	pack .select.f -fill both -expand y
	wm resizable .select 0 0
}
proc ListSelectMP3 { channel filestring } {
	global mp3tool connections
	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if [winfo exists .select] {destroy .select}
	toplevel .select
	wm title .select [Trans lister_window_selection]
	button .select.a -text [Trans request_file] -command {ListREALSelectMP3 $mp3tool(chan) [selection get]} -activeforeground red
 	button .select.b -text [Trans request_list] -command {RequestList [selection get]} -activeforeground red
 	button .select.c -text [Trans request_que] -command {RequestQue [selection get]} -activeforeground red
 	button .select.e -text [Trans request_status] -command {RequestQueStatus [selection get]} -activeforeground red
	button .select.d -text [Trans request_remove] -command {RequestRemove [selection get]} -activeforeground red
	button .select.f -text [Trans ignore_user] -command {RequestIgnore [selection get]} -activeforeground red
	button .select.g -text [Trans add_to_wish_list] -command {ListWISHSelectMP3 [selection get]} -activeforeground red
	pack .select.a -fill both -expand y
	pack .select.b -fill both -expand y
	pack .select.c -fill both -expand y
	pack .select.e -fill both -expand y
	pack .select.d -fill both -expand y
	pack .select.f -fill both -expand y
	pack .select.g -fill both -expand y
	wm resizable .select 0 0
}

proc WishSelectMP3 { channel filestring } {
	global mp3tool connections
	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if [winfo exists .select] {destroy .select}
	toplevel .select
	wm title .select [Trans lister_window_selection]
	button .select.a -text [Trans request_file] -command {WishREALSelectMP3 $mp3tool(chan) [selection get]} -activeforeground red
 	button .select.b -text [Trans request_list] -command {RequestList [selection get]} -activeforeground red
 	button .select.c -text [Trans request_que] -command {RequestQue [selection get]} -activeforeground red
 	button .select.e -text [Trans request_status] -command {RequestQueStatus [selection get]} -activeforeground red
	button .select.d -text [Trans request_remove] -command {RequestRemove [selection get]} -activeforeground red
	button .select.f -text [Trans ignore_user] -command {RequestIgnore [selection get]} -activeforeground red
	button .select.g -text [Trans remove_from_wish_list] -command {RemoveFromWishList} -activeforeground red
	pack .select.a -fill both -expand y
	pack .select.b -fill both -expand y
	pack .select.c -fill both -expand y
	pack .select.e -fill both -expand y
	pack .select.d -fill both -expand y
	pack .select.f -fill both -expand y
	pack .select.g -fill both -expand y
	wm resizable .select 0 0
}

proc RemoveFromWishList {} {
	global mp3tool 
	if [winfo exists .select] {destroy .select}
	.wmp3.l.list delete [.wmp3.l.list curselection]
	set outfile [open $mp3tool(dir)/WishList.txt w]
	for {set c 0} {$c < [.wmp3.l.list index end]} {incr c} {
		puts $outfile [string range [.wmp3.l.list get $c $c] 1 [expr [string length [.wmp3.l.list get $c $c]] -1]]
	}
	close $outfile
}

proc RequestList {filestring} {
	global mp3tool
	if {[winfo exists .select]} {destroy .select}
	set whoall [string range $filestring [expr [string first "!" $filestring] +1] end]
	set who [string range $whoall 0 [expr [string first " " $whoall] -1]]
	SendMSG $mp3tool(chan) "@$who"
}

proc RequestIgnore {filestring} {
	global mp3tool
	if {[winfo exists .select]} {destroy .select}
	set whoall [string range $filestring [expr [string first "!" $filestring] +1] end]
	set who [string range $whoall 0 [expr [string first " " $whoall] -1]]
	SendIGNORE $who
}

proc RequestQue {filestring} {
	global mp3tool
	if {[winfo exists .select]} {destroy .select}
	set whoall [string range $filestring [expr [string first "!" $filestring] +1] end]
	set who [string range $whoall 0 [expr [string first " " $whoall] -1]]
	SendMSG $mp3tool(chan) "@$who-que"
}

proc RequestQueStatus {filestring} {
	global mp3tool
	if {[winfo exists .select]} {destroy .select}
	set whoall [string range $filestring [expr [string first "!" $filestring] +1] end]
	set who [string range $whoall 0 [expr [string first " " $whoall] -1]]
	SendMSG $mp3tool(chan) "@$who-stats"
}

proc RequestRemove {filestring} {
	global mp3tool
	if {[winfo exists .select]} {destroy .select}
	set whoall [string range $filestring [expr [string first "!" $filestring] +1] end]
	set who [string range $whoall 0 [expr [string first " " $whoall] -1]]
	SendMSG $mp3tool(chan) "@$who-remove"
}

proc ListREALSelectMP3 { channel filestring } {
	global currentNet recent mp3tool recentcount connections

	if {[winfo exists .select]} {destroy .select}

	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3 Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}

	.lmp3.l.list delete [.lmp3.l.list curselection]
	SendMSG $channel [string range $filestring [string first "!" $filestring] end]

	incr mp3tool(total) -1
	if {$recentcount > $mp3tool(maxrecent)} {
		for {set c 1} {$c <= $mp3tool(maxrecent)} {incr c} {
			set recent([expr $c -1]) $recent($c)
		}
		set recent($mp3tool(maxrecent)) "\[[clock format [clock seconds] -format "%T"]\] [string range $filestring [string first "!" $filestring] end]"
	} else {
		set recent($recentcount) "\[[clock format [clock seconds] -format "%T"]\] [string range $filestring [string first "!" $filestring] end]"
		incr recentcount
	}		
}

proc WishREALSelectMP3 { channel filestring } {
	global currentNet recent mp3tool recentcount connections

	if {[winfo exists .select]} {destroy .select}

	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3 Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}

	.wmp3.l.list delete [.wmp3.l.list curselection]
	SendMSG $channel [string range $filestring [string first "!" $filestring] end]

	incr mp3tool(total) -1
	if {$recentcount > $mp3tool(maxrecent)} {
		for {set c 1} {$c <= $mp3tool(maxrecent)} {incr c} {
			set recent([expr $c -1]) $recent($c)
		}
		set recent($mp3tool(maxrecent)) "\[[clock format [clock seconds] -format "%T"]\] [string range $filestring [string first "!" $filestring] end]"
	} else {
		set recent($recentcount) "\[[clock format [clock seconds] -format "%T"]\] [string range $filestring [string first "!" $filestring] end]"
		incr recentcount
	}		
}


proc ListWISHSelectMP3 { filestring } {
	global mp3tool

	if {[winfo exists .select]} {destroy .select}

	.lmp3.l.list delete [.lmp3.l.list curselection]

	set outfile [open $mp3tool(dir)/WishList.txt a]
	puts $outfile "$filestring"
	close $outfile
	if [winfo exists .wmp3.l.list] {
		.wmp3.l.list insert end $filestring
	}
}


proc WISHSelectMP3 { filestring } {
	global mp3tool

	if {[winfo exists .select]} {destroy .select}

	.mp3.l.list delete [.mp3.l.list curselection]
	
	set outfile [open $mp3tool(dir)/WishList.txt a]
	puts $outfile "[string range $filestring [string first "!" $filestring] end]"
	close $outfile
	if [winfo exists .wmp3.l.list] {
		.wmp3.l.list insert end "[string range $filestring [string first "!" $filestring] end]"
	}
}

proc REALSelectMP3 { channel filestring } {
	global currentNet recent mp3tool recentcount connections

	if {[winfo exists .select]} {destroy .select}

	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}

	.mp3.l.list delete [.mp3.l.list curselection]
	SendMSG $channel [string range $filestring [string first "!" $filestring] end]

	incr mp3tool(total) -1
	if {$recentcount > $mp3tool(maxrecent)} {
		for {set c 1} {$c <= $mp3tool(maxrecent)} {incr c} {
			set recent([expr $c -1]) $recent($c)
		}
		set recent($mp3tool(maxrecent)) "\[[clock format [clock seconds] -format "%T"]\] [string range $filestring [string first "!" $filestring] end]"
	} else {
		set recent($recentcount) "\[[clock format [clock seconds] -format "%T"]\] [string range $filestring [string first "!" $filestring] end]"
		incr recentcount
	}		
}


proc SelectMP3R { channel filestring } {
	global mp3tool

	if [winfo exists .select] {destroy .select}
	toplevel .select
	wm title .select [Trans recycle_window_selection]
	button .select.a -text [Trans request_file] -command {REALSelectMP3R $mp3tool(chan) [selection get]} -activeforeground red
 	button .select.b -text [Trans request_list] -command {RequestList [selection get]} -activeforeground red
 	button .select.c -text [Trans request_que] -command {RequestQue [selection get]} -activeforeground red
 	button .select.e -text [Trans request_status] -command {RequestQueStatus [selection get]} -activeforeground red
	button .select.d -text [Trans request_remove] -command {RequestRemove [selection get]} -activeforeground red
	button .select.f -text [Trans ignore_user] -command {RequestIgnore [selection get]} -activeforeground red
	button .select.g -text [Trans add_to_wish_list] -command {WISHSelectMP3R [selection get]} -activeforeground red
	pack .select.a -fill both -expand y
	pack .select.b -fill both -expand y
	pack .select.c -fill both -expand y
	pack .select.e -fill both -expand y
	pack .select.d -fill both -expand y
	pack .select.f -fill both -expand y
	pack .select.g -fill both -expand y
	wm resizable .select 0 0
}

proc REALSelectMP3R { channel filestring } {
	if {[winfo exists .select]} {destroy .select}
	SendMSG $channel [string range $filestring [string first "!" $filestring] end]
}

proc WISHSelectMP3R { filestring } {
	global mp3tool
	if {[winfo exists .select]} {destroy .select}

	set outfile [open $mp3tool(dir)/WishList.txt a]
	puts $outfile "[string range $filestring [string first "!" $filestring] end]"
	close $outfile
	if [winfo exists .wmp3.l.list] {
		.wmp3.l.list insert end "[string range $filestring [string first "!" $filestring] end]"
	}
}

proc HasListFile { file } {
	global sendlist slist search_index single_search_index mp3tool
	if {[string range $file 0 0] == "!"} {
		set file [string range $file [expr [string first " " $file] +1] end]
	}
	set file [string tolower $file]
	set thr [string range $file 0 2]
	if {[info exists search_index([string range $file 0 1])]} {
		set d $search_index([string range $file 0 1])
	} else {
		if {[info exists single_search_index([string range $file 0 0])]} {
			set d $single_search_index([string range $file 0 0])
		} else {
			return "--"
		}
	}
	for {set c $d} {$c < $mp3tool(totalfiles)} {incr c} {
		if {[string range $slist($c) 0 2] > $thr} {
			return "--"
		}
		if {$file == $slist($c)} {
			return $file
		}
	}
	return "--"
}

proc HasFile { file } {
	global mp3dirs mp3tool

	# First try the caches, this should speed things up quite a bit
	# when new entries come in.
	if {[set idx [lsearch -exact $mp3tool(bad_cache) "[string tolower $file]"]] != -1} {
		return "--"
	}
	foreach dir $mp3dirs {
		if {[set idx [lsearch -exact $mp3tool(good_cache) "$dir/$file"]] != -1} {
			return "$dir/$file"
		}
		if {[set idx [lsearch -exact $mp3tool(good_cache) "$dir/[string tolower $file]"]] != -1} {
			return "$dir/[string tolower $file]"
		}
	}
	set dir ""
	foreach dir $mp3dirs {
		if {[file exists "$dir/$file"]} {
			lappend mp3tool(good_cache) "$dir/$file"
			return "$dir/$file"
		}
		if {[file exists "$dir/[string tolower $file]"]} {
			lappend mp3tool(good_cache) "$dir/[string tolower $file]"
			return "$dir/[string tolower $file]"
		}
	}
	lappend mp3tool(bad_cache) "[string tolower $file]"
	return "--"
}

proc PlaySelected { file } {
	global mp3tool
	exec $mp3tool(x11amp) $file &
}

proc LockTog { } {
	global scrlk

	if { $scrlk } {
		set scrlk 0
		.mp3.l.scrl.lk configure -image unlocked
	} else {
		set scrlk 1
		.mp3.l.scrl.lk configure -image locked
	}
}

proc ChlstTog { } {
	global mp3tool

	if { $mp3tool(chlst) == 2} {
		set mp3tool(chlst) 0
		.mp3.ch.in.srch.chlst configure -text [Trans list]
	} elseif { $mp3tool(chlst) == 1} {
		set mp3tool(chlst) 2
		.mp3.ch.in.srch.chlst configure -text [Trans ctcp]
	} else {
		set mp3tool(chlst) 1
		.mp3.ch.in.srch.chlst configure -text [Trans channel]
	}
}

proc SearchMP3List { what } {
	global mp3tool

	if { $mp3tool(chlst) == 1} {
		SendMSG $mp3tool(chan) "@find $what"
		return
	} elseif { $mp3tool(chlst) == 2} {
		SendMSG $mp3tool(chan) "@ctcpfind $what"
		return
	}
	set d 0
	for {set c 0} {$c <= [.mp3.l.list index end]} {incr c} {
		if { [.mp3.l.list selection includes $c] } {
			.mp3.l.list selection clear $c
			set d [expr $c + 1]
			break
		}
	}
	for {set c $d} {$c <= [.mp3.l.list index end]} {incr c} {
		if {[string first [string tolower $what] [string tolower [.mp3.l.list get $c]]] != -1} {
			.mp3.l.list selection set $c
			.mp3.l.list see $c
			return
		}
	}
}

proc SearchTXTList { what } {
	set d 0
	for {set c 0} {$c <= [.lmp3.l.list index end]} {incr c} {
		if { [.lmp3.l.list selection includes $c] } {
			.lmp3.l.list selection clear $c
			set d [expr $c + 1]
			break
		}
	}
	for {set c $d} {$c <= [.lmp3.l.list index end]} {incr c} {
		if {[string first [string tolower $what] [string tolower [.lmp3.l.list get $c]]] != -1} {
			.lmp3.l.list selection set $c
			.lmp3.l.list see $c
			return
		}
	}
}

proc SearchWISHList { what } {
	set d 0
	for {set c 0} {$c <= [.wmp3.l.list index end]} {incr c} {
		if { [.wmp3.l.list selection includes $c] } {
			.wmp3.l.list selection clear $c
			set d [expr $c + 1]
			break
		}
	}
	for {set c $d} {$c <= [.wmp3.l.list index end]} {incr c} {
		if {[string first [string tolower $what] [string tolower [.wmp3.l.list get $c]]] != -1} {
			.wmp3.l.list selection set $c
			.wmp3.l.list see $c
			return
		}
	}
}

proc SearchOwnTXTList { what } {
	set d 0
	for {set c 0} {$c <= [.olmp3.l.list index end]} {incr c} {
		if { [.olmp3.l.list selection includes $c] } {
			.olmp3.l.list selection clear $c
			set d [expr $c + 1]
			break
		}
	}
	for {set c $d} {$c <= [.olmp3.l.list index end]} {incr c} {
		if {[string first [string tolower $what] [string tolower [.olmp3.l.list get $c]]] != -1} {
			.olmp3.l.list selection set $c
			.olmp3.l.list see $c
			return
		}
	}
}

proc SendChan { what } {
	global mp3tool
	
	SendMSG $mp3tool(chan) [subst $what]
	lappend mp3tool(commandlist) $what
	if [winfo exists .send] {
		destroy .send
	}
}

proc SendToChan { } {
	global mp3tool connections tcl_platform

	if {$mp3tool(chan) == "--SETMEUP--"} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans you_have_not_entered_a_channel_name]
		label .error.ll -text [Trans enter_one_in_the_main_window]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if {$connections == 0} {
		if [winfo exists .error] {destroy .error}
		toplevel .error
		wm title .error "MP3Tool [Trans by] Cruise - [Trans error]!!"
		ErrorIcon .error
		label .error.l -text [Trans mp3tool_is_not_connected]
		label .error.ll -text [Trans type_mp3start_in_your_irc_client]
		button .error.b -text [Trans ok] -command {destroy .error}
		pack .error.l
		pack .error.ll
		pack .error.b
		return
	}
	if ![winfo exists .send] {
		toplevel .send
		wm title .send "MP3Tool [Trans by] Cruise - [Trans sending_to] $mp3tool(chan)"
		set platform [array get tcl_platform platform]
		if {$platform == "platform unix"} {
			wm iconbitmap .send @$mp3tool(dir)/images/send.xbm
		}
		entry .send.what -width 50 -relief sunken
		button .send.send -text [Trans send]  -bg "light blue" \
			-activeforeground red \
                      -activebackground "light blue" \
			-command { SendChan [.send.what get] }
		pack .send.send -side left
		pack .send.what -side right -fill both -expand y
		bind .send <Return> { SendChan [.send.what get] }
		bind .send <Up> {PrevCommand .send}
		bind .send <Down> {NextCommand .send}
	} else {
		destroy .send
		update
		SendToChan
		return
	}
}

proc Recycle { } {
	global mp3tool recent recentcount tcl_platform

	if ![winfo exists .recycle] {
		toplevel .recycle
		wm title .recycle "MP3Tool [Trans by] Cruise - [Trans relisting_requests_for] $mp3tool(chan)"
		frame .recycle.l
		frame .recycle.x
		listbox .recycle.l.list -borderwidth 2 -height 5 -width 60 \
			-yscrollcommand ".recycle.l.scrolly set" \
			-xscrollcommand ".recycle.x.scrollx set"
		scrollbar .recycle.l.scrolly -command ".recycle.l.list yview"
		scrollbar .recycle.x.scrollx -command ".recycle.l.list xview" \
                        -orient horizontal
		button .recycle.x.recycle -image recycle -command Recycle 
		button .recycle.x.send -image send -command SendToChan
		pack .recycle.x -side top -fill both
		pack .recycle.l -expand y -fill both
		pack .recycle.l.list .recycle.l.scrolly -side left -fill y -expand y
		bind .recycle.l.list <ButtonRelease> { SelectMP3R [.mp3.ch.in.ch get] [selection get] }
		pack .recycle.x.recycle .recycle.x.send -side right
		pack .recycle.x.scrollx -side left -fill x -expand y
		wm resizable .recycle 0 1
		set platform [array get tcl_platform platform]
		if {$platform == "platform unix"} {
			wm iconbitmap .recycle @$mp3tool(dir)/images/recycle.xbm
		}
	} else {
		destroy .recycle
		update
		Recycle
		return
	}
	for {set c 0} {$c < $recentcount} {incr c} {
		.recycle.l.list insert end $recent($c)
	}
}

proc GetWindowSetup { } {
	global scrlk mp3tool sends
	global configfile mp3tool tcl_platform


	if ![winfo exists .mp3] {
		set platform [array get tcl_platform platform]
		if {$mp3tool(totalfiles) == -1} {SetVars}
		image create photo unlocked -file $mp3tool(dir)/images/unlock.gif
		image create photo locked -file $mp3tool(dir)/images/lock.gif
		image create photo recycle -file $mp3tool(dir)/images/recycle.gif
		image create photo send -file $mp3tool(dir)/images/send.gif
		image create photo shutdown -file $mp3tool(dir)/images/shutdown.gif
		image create photo server -file $mp3tool(dir)/images/server.gif
		image create photo help -file $mp3tool(dir)/images/help.gif
		image create photo configure -file $mp3tool(dir)/images/configure.gif
		image create photo nonew -file $mp3tool(dir)/images/nonew.gif
		image create photo yesnew -file $mp3tool(dir)/images/yesnew.gif
		image create photo dump -file $mp3tool(dir)/images/dump.gif
		image create photo mouth -file $mp3tool(dir)/images/mouth.gif
		image create photo play -file $mp3tool(dir)/images/play.gif
		image create photo sm_play -file $mp3tool(dir)/images/sm_play.gif
		image create photo choose -file $mp3tool(dir)/images/choose.gif
		image create photo listviewer -file $mp3tool(dir)/images/listviewer.gif
		image create photo floppy -file $mp3tool(dir)/images/floppy.gif
		image create photo pinfo -file $mp3tool(dir)/images/info.gif
		image create photo onleft -file $mp3tool(dir)/images/onleft.gif
		image create photo onright -file $mp3tool(dir)/images/onright.gif
		image create photo offleft -file $mp3tool(dir)/images/offleft.gif
		image create photo offright -file $mp3tool(dir)/images/offright.gif
		image create photo wait1 -file $mp3tool(dir)/images/wait1.gif
		image create photo wait2 -file $mp3tool(dir)/images/wait2.gif
		image create photo wait3 -file $mp3tool(dir)/images/wait3.gif
		image create photo wait4 -file $mp3tool(dir)/images/wait4.gif
		image create photo wait5 -file $mp3tool(dir)/images/wait5.gif
		image create photo wait6 -file $mp3tool(dir)/images/wait6.gif
		image create photo wait7 -file $mp3tool(dir)/images/wait7.gif
		image create photo wait8 -file $mp3tool(dir)/images/wait8.gif
		image create photo wait9 -file $mp3tool(dir)/images/wait9.gif
		image create photo wait10 -file $mp3tool(dir)/images/wait10.gif
		image create photo wait11 -file $mp3tool(dir)/images/wait11.gif
		image create photo wait12 -file $mp3tool(dir)/images/wait12.gif
		image create photo wait13 -file $mp3tool(dir)/images/wait13.gif
		image create photo wait14 -file $mp3tool(dir)/images/wait14.gif
		image create photo wait15 -file $mp3tool(dir)/images/wait15.gif
		image create photo wait16 -file $mp3tool(dir)/images/wait16.gif
		image create photo wait17 -file $mp3tool(dir)/images/wait17.gif
		image create photo wait18 -file $mp3tool(dir)/images/wait18.gif
		image create photo wait19 -file $mp3tool(dir)/images/wait19.gif
		image create photo wish -file $mp3tool(dir)/images/wish.gif
 		set mp3tool(total) 0
		set scrlk 0
		toplevel .mp3
		wm title .mp3 "MP3Tool For $mp3tool(client) v$mp3tool(version) [Trans by] Cruise <cruise@drunkenbastards.org>"
		frame .mp3.pl -relief sunken -bg black
		button .mp3.pl.play -image play -command { PlaySelected [.mp3.pl.entry get] }
		entry .mp3.pl.entry -width 62 -relief sunken -bg black \
			-fg "light green"
		frame .mp3.l -relief sunken
		listbox .mp3.l.list -borderwidth 2 -height 5 -width 65\
			-yscrollcommand ".mp3.l.scrl.scroll set" \
			-xscrollcommand ".mp3.scrlx.scrollx set"
		frame .mp3.l.scrl
		frame .mp3.scrlx
		scrollbar .mp3.scrlx.scrollx -command ".mp3.l.list xview" \
			-orient horizontal
		scrollbar .mp3.l.scrl.scroll -command ".mp3.l.list yview"
		button .mp3.l.scrl.lk -image unlocked -command LockTog
		button .mp3.scrlx.recycle -image recycle -command Recycle
		button .mp3.scrlx.send -image send -command SendToChan
		button .mp3.scrlx.wish -image wish -command WishList
		button .mp3.scrlx.pinfo -image pinfo -command SLOTSInfo
		button .mp3.scrlx.shutdown -image shutdown -command StopMP3Tool
		button .mp3.scrlx.help -image help -command MP3ToolHelp
		button .mp3.scrlx.configure -image configure -command DrawConfigureWindow
		button .mp3.scrlx.server -image server -command DrawServerWindow
		frame .mp3.scrlx.act -relief raised -borderwidth 2
		label .mp3.scrlx.act.r -image offright
		label .mp3.scrlx.act.l -image offleft
		frame .mp3.ch -relief sunken -bg "light blue"
		frame .mp3.ch.in -relief raised  -bg "light blue"
		label .mp3.ch.in.l -text [Trans channel_name] -relief raised \
			-bg "light blue"
		entry .mp3.ch.in.ch -relief sunken -width 20
		frame .mp3.ch.in.srch -bg "light blue"
		button .mp3.ch.in.srch.listviewer -image listviewer \
			-command "SelectList"
		button .mp3.ch.in.srch.chlst -text [Trans channel] -command ChlstTog \
			-bg "light blue" -activeforeground red \
                        -activebackground "light blue"
		button .mp3.ch.in.srch.sl -relief raised -text [Trans search_for] \
			-bg "light blue" -activeforeground red \
			-activebackground "light blue" \
			-command { SearchMP3List [.mp3.ch.in.srch.entry get] }
		entry .mp3.ch.in.srch.entry -relief sunken -width 20		
		scale .mp3.ch.slider -orient horizontal -from 1 -to 500 \
			-variable mp3tool(maxmp3s) -label [Trans max_entries] \
			-relief raised -bg "light blue" \
			-highlightbackground "light blue" -highlightcolor "red"
		pack .mp3.pl .mp3.ch -side top -fill both
		pack .mp3.l -side bottom -fill both -expand y
		pack .mp3.scrlx -side bottom -fill both
		pack .mp3.scrlx.act .mp3.scrlx.recycle .mp3.scrlx.send .mp3.scrlx.wish .mp3.scrlx.pinfo .mp3.scrlx.server .mp3.scrlx.help .mp3.scrlx.configure .mp3.scrlx.shutdown -side right
		pack .mp3.scrlx.act.r .mp3.scrlx.act.l -side left -expand y
		pack .mp3.scrlx.scrollx -side left -fill x -expand y
		pack .mp3.ch.in -side left -fill both -expand y
		pack .mp3.ch.in.srch -side bottom -fill both -expand y
		pack .mp3.ch.in.l .mp3.ch.in.ch -side left -fill both -expand y
		pack .mp3.ch.in.srch.listviewer .mp3.ch.in.srch.chlst .mp3.ch.in.srch.sl .mp3.ch.in.srch.entry -side left -fill both -expand y 
		pack .mp3.ch.slider -side right -fill x 
		pack .mp3.l.list .mp3.l.scrl -side left -fill both
		pack .mp3.l.scrl.lk -side bottom
		pack .mp3.l.scrl.scroll -side top -fill y -expand y
		pack .mp3.pl.play .mp3.pl.entry -side left -fill y -expand y
		bind .mp3.l.list <ButtonRelease> { SelectMP3 [.mp3.ch.in.ch get] [selection get] }
		bind .mp3.ch.in.ch <KeyRelease> { set mp3tool(chan) [.mp3.ch.in.ch get]}
		bind .mp3.ch.in.srch.listviewer <Button-2> SelectOwnList
		bind .mp3.ch.in.srch.listviewer <Button-3> SelectOwnList
		wm resizable .mp3 0 1
		if {$platform == "platform unix"} {
			wm iconbitmap .mp3 @$mp3tool(dir)/images/yesnew.xbm
		}
		if {$mp3tool(client) == "Zircon"} {
			if {$mp3tool(lineup) == ""} {
				.mp3.ch.in.ch insert end $mp3tool(chan)
			} {
				.mp3.ch.in.ch insert end $mp3tool(lineup)
			}
		} {
			if {$mp3tool(lineup) != ""} {
				.mp3.ch.in.ch insert end $mp3tool(lineup)
				set mp3tool(chan) $mp3tool(lineup)
			}
		}
	}
}

proc SLOTSInfo {} {
	global mp3tool slotsend slotsavail slottime slotque slotqavail
	global slotexp slotfiles slotcps tcl_platform slotname
	
	if {[winfo exists .slots]} {
		destroy .slots
		update
	}
	toplevel .slots
	wm title .slots "MP3Tool [Trans user_slots_info]"
	set platform [array get tcl_platform platform]
	if {$platform == "platform unix"} {
		wm iconbitmap .slots @$mp3tool(dir)/images/info.xbm
		set fpad 24
		set twid 96
	} else {
		set fpad 12
		set twid 96
	}
	set seconds [clock seconds]
	frame .slots.top
	label .slots.top.1 -width 12 -bg "light blue" -text [Trans username] -relief sunken -borderwidth 2
	label .slots.top.2 -width 10 -bg "light blue" -text [Trans send_total] -relief sunken -borderwidth 2
	label .slots.top.3 -width 10 -bg "light blue" -text [Trans send_avail] -relief sunken -borderwidth 2
	label .slots.top.4 -width 10 -bg "light blue" -text [Trans free_in] -relief sunken -borderwidth 2
	label .slots.top.5 -width 10 -bg "light blue" -text [Trans que_total] -relief sunken -borderwidth 2
	label .slots.top.6 -width 10 -bg "light blue" -text [Trans que_inuse] -relief sunken -borderwidth 2
	label .slots.top.7 -width 10 -bg "light blue" -text [Trans cps_inuse] -relief sunken -borderwidth 2
	label .slots.top.8 -width 15 -bg "light blue" -text [Trans total_files] -relief sunken -borderwidth 2
	frame .slots.f 
	text .slots.f.text -height 10 -width $twid -state disabled \
		-relief sunken -borderwidth 2 -yscrollcommand ".slots.f.yscroll set"
	scrollbar .slots.f.yscroll -command ".slots.f.text yview"

	for {set c 1} {$c <= $mp3tool(totalslots)} {incr c} {
		if {$slotexp($c) > $seconds} {
			frame .slots.f.text.$c
			button .slots.f.text.$c.but$c -text $slotname($c) -command "InfoButton $c" -width 12
			label .slots.f.text.$c.send$c -text $slotsend($c) -width 10 -bg "light blue" -relief sunken -borderwidth 2
			label .slots.f.text.$c.savail$c -text $slotsavail($c) -width 10 -bg "light blue" -relief sunken -borderwidth 2
			label .slots.f.text.$c.time$c -text $slottime($c) -width 10 -bg "light blue" -relief sunken -borderwidth 2
			label .slots.f.text.$c.qye$c -text $slotque($c) -width 10 -bg "light blue" -relief sunken -borderwidth 2
			label .slots.f.text.$c.qavail$c -text $slotqavail($c) -width 10 -bg "light blue" -relief sunken -borderwidth 2
			label .slots.f.text.$c.cps$c -text $slotcps($c) -width 10 -bg "light blue" -relief sunken -borderwidth 2
			label .slots.f.text.$c.files$c -text [string range $slotfiles($c) 0 14] -width 15 -bg "light blue" -relief sunken -borderwidth 2
		}
	}

	pack .slots.top -fill both -side top -padx $fpad
	pack .slots.top.1 .slots.top.2 .slots.top.3 .slots.top.4 .slots.top.5 .slots.top.6 .slots.top.7 .slots.top.8 -side left
	pack .slots.f -fill both -expand y -side bottom
	pack .slots.f.text .slots.f.yscroll -fill both -expand y -side left
	pack .slots.f.yscroll -fill y -expand y -side right
	for {set c 1} {$c <= $mp3tool(totalslots)} {incr c} {
		if {$slotexp($c) > $seconds} {
			pack .slots.f.text.$c -fill y -expand y
			pack .slots.f.text.$c.but$c .slots.f.text.$c.send$c .slots.f.text.$c.savail$c .slots.f.text.$c.time$c .slots.f.text.$c.qye$c .slots.f.text.$c.qavail$c .slots.f.text.$c.cps$c  .slots.f.text.$c.files$c -side left -fill both -expand y
			.slots.f.text window create [expr $c -1].0 -window .slots.f.text.$c
		}
	}
	wm resizable .slots 0 1

}

proc SetVars {} {
	global filelist totfin currentNet quepercent
	global grand_date mp3dirs serverdirs sends
	global mp3tool tcl_platform uservar questatus INTwords excludes
	global findtimer argv

	set mp3tool(version) "2.0"
	set mp3tool(totalfiles) 0
	set totfin 0
	set sends(TIMERFLAG) 0
	set mp3tool(ap_blink) 0
	set mp3tool(usedslots) 0
	set mp3tool(quep) 0
	set mp3tool(sfreq) 0
	set mp3tool(distribute) 0
	set mp3tool(total) 0
	set mp3tool(debugger) 0
	set sends(SPECIAL) ""
	set sends(TIMEOUT) 0
	set mp3tool(SPECIAL) 0
	set mp3tool(totalfiles) -1
	set mp3tool(good_cache) { }
	set mp3tool(bad_cache) { }
	set mp3tool(language) "English"
	set mp3tool(lineup) ""
	set mp3tool(commandlist) { }
	set mp3tool(autoplay) 0
	set findtimer(0) "0"

	for {set c 0} {$c <= 20} {incr c} {set uservar($c) 0}
	for {set c 0} {$c <= 30} {incr c} {
		set questatus($c) "Idle"
		set quepercent($c) 0
	}

        set platform [array get tcl_platform platform]
	switch -exact $mp3tool(client) {
		"OpenVerse"	{
			global MV
			set mp3tool(dir) "$MV(homedir)/plugins/MP3Tool"
        	        set mp3tool(configfile) "$mp3tool(dir)/MP3Tool.cfg"
			set nick $MV(nick)
		}
		"Zircon"	{
			global zircon
			set mp3tool(dir) "$zircon(lib)/plugins/MP3Tool"
        		if {$platform == "platform unix"} {
        	        	set mp3tool(configfile) "~/.zircon/MP3Tool.config"
        		} else {
                		set mp3tool(configfile) "$zircon(lib)/plugins/MP3Tool.config"
        		}
			set nick [$currentNet nickname]
		}
		"BitchX"	{
        		if {$platform == "platform unix"} {
				set mp3tool(dir) "~/.BitchX/MP3Tool"
	                	set mp3tool(configfile) "~/.BitchX/MP3Tool/MP3Tool.config"
        		} else {
				#FIXME
				# Only UNIX  is supported at the moment.
				# EXit with an error here.
        		}
			set mp3tool(nick) BitchX
			set nick BitchX
		}
		"epic ircII"	{
        		if {$platform == "platform unix"} {
				set mp3tool(dir) "~/MP3Tool"
	                	set mp3tool(configfile) "~/MP3Tool/MP3Tool.config"
        		} else {
				#FIXME
				# Only UNIX BitchX is supported at the moment.
				# EXit with an error here.
        		}
			set mp3tool(nick) epicirc
			set nick epicirc
			set sends(TIMEOUT) 1
		}
		"mIRC"	{
	        	set mp3tool(configfile) "$mp3tool(dir)/MP3Tool.cfg"
			set mp3tool(nick) mIRC
			set nick mIRC
		}
		default {#Give An Errror}
	}

	# If they gave an argument, use it instead of the stock config.
	if {$argv != ""} {
		set mp3tool(configfile) "$mp3tool(dir)/$argv"
	}

	if [file exists $mp3tool(configfile)] {
		source $mp3tool(configfile)
	} else {
		set outfile [open $mp3tool(configfile) "w"]
		puts $outfile "set sends(CTCP) 1"
		puts $outfile "set sends(CHAN) 1"
		puts $outfile "set sends(FIND) 1"
		puts $outfile "set sends(COLOR) 1"
		puts $outfile "set sends(LIST) 1"
		puts $outfile "set sends(MAX_FIND) 5"
		puts $outfile "set sends(STYLE) \"Default\""
		puts $outfile "set mp3tool(quemax) 12"
		puts $outfile "set mp3tool(sendmax) 3"
		puts $outfile "set mp3tool(MAXINSEND) 1"
		puts $outfile "set mp3tool(MAXINQUE) 1"
	        if {$platform == "platform unix"} {
			puts $outfile "set mp3tool(x11amp) \"x11amp\""
		} else {
			puts $outfile "set mp3tool(x11amp) \"winamp.exe\""
		}
		puts $outfile "set mp3tool(maxmp3s) 20"
		puts $outfile "set mp3tool(maxrecent) 10"
		puts $outfile "set mp3tool(freq) 3"
	        if {$platform == "platform unix"} {
			puts $outfile "set filelist \"~/$nick.txt\""
		} else {
			puts $outfile "set filelist \"$mp3tool(dir)/$nick.txt\""
		}
	        if {$platform == "platform unix"} {
				puts $outfile "set  mp3tool(grand_file) \"~/TotalMP3sSent\""
		} else {
				puts $outfile "set mp3tool(grand_file) \"$mp3tool(dir)/TotalMP3sSent\""
		}
		puts $outfile "set grand_date \"[clock format [clock seconds]]\""
		puts $outfile "set mp3dirs { }"
		puts $outfile "set serverdirs { }"
		close $outfile
		source $mp3tool(configfile)
	}
	source "$mp3tool(dir)/languages/English"
	source "$mp3tool(dir)/languages/$mp3tool(language)"
	set excludes { }
	if [file exists $mp3tool(dir)/Ecludes] {
		set infile [open $mp3tool(dir)/Excludes r]
		while {[eof $infile] != 1} {
			gets $infile input
			if {[string length $input] > 3} {
				lappend excludes $input
			}
		}
		close $infile
	}
}

proc SelectList {} {
	MP3mkFileBox .browse ".*.txt" ".*.txt" [Trans mp3_list_viewer_tool] [Trans select_list_to_view] "" "{[Trans choose_file]} ListerSetup" "{[Trans dismiss]} DismissedWindow" 
}

proc DismissedWindow {stuff} {
  # Do Absoutly Nothing.
}

proc SelectOwnList {} {
	global mp3tool filelist tcl_platform

	if [winfo exists .olmp3] {
		destroy .olmp3
	}
	toplevel .olmp3
	wm title .olmp3 "MP3Tool [Trans list_yours_browser]"
	wm geometry .olmp3 484x140
	set platform [array get tcl_platform platform]
	if {$platform == "platform unix"} {
		wm iconbitmap .olmp3 @$mp3tool(dir)/images/listviewer.xbm
	}
	frame .olmp3.l -relief sunken
	listbox .olmp3.l.list -borderwidth 2 -height 5 -width 1\
		-yscrollcommand ".olmp3.l.scroll set" \
		-xscrollcommand ".olmp3.scrlx.scrollx set"
	frame .olmp3.scrlx
	scrollbar .olmp3.scrlx.scrollx -command ".olmp3.l.list xview" \
		-orient horizontal
	scrollbar .olmp3.l.scroll -command ".olmp3.l.list yview"
	frame .olmp3.ch -relief sunken -bg "light blue"
	frame .olmp3.ch.in -relief raised  -bg "light blue"
	frame .olmp3.ch.in.srch -bg "light blue"
	button .olmp3.ch.in.srch.sl -relief raised -text [Trans search_for] \
		-bg "light blue" -activeforeground red \
		-activebackground "light blue" \
		-command { SearchOwnTXTList [.olmp3.ch.in.srch.entry get] }
	entry .olmp3.ch.in.srch.entry -relief sunken -width 20		
	set infile [open $filelist r]
	set count 0
	while {[eof $infile] != 1} {
		gets $infile input
		set brkpnt [string first ".mp3" [string tolower $input]]
		if {$brkpnt >= 4} {
			.olmp3.l.list insert end [string range $input 0 [expr $brkpnt +3]]
			incr count
		}
	}
	close $infile
	label .olmp3.ch.in.srch.info -relief raised -borderwidth 2 -bg "light blue" -text "[Trans this_is_your_list] ($count [Trans files])"
	pack .olmp3.ch -side top -fill both
	pack .olmp3.l -side bottom -fill both -expand y
	pack .olmp3.scrlx -side bottom -fill both
	pack .olmp3.scrlx.scrollx -side left -fill x -expand y
	pack .olmp3.ch.in -side left -expand y -fill both
	pack .olmp3.ch.in.srch -side bottom -expand y -fill both
	pack .olmp3.ch.in.srch.sl .olmp3.ch.in.srch.entry .olmp3.ch.in.srch.info -side left -fill both -expand y
	pack .olmp3.l.list -side left -fill both -expand y
	pack .olmp3.l.scroll -side top -fill y -expand y
	bind .olmp3.ch.in.srch.entry <Return> { SearchOwnTXTList [.olmp3.ch.in.srch.entry get]}
	bind .olmp3.l.list <ButtonRelease> {SelectOwn [selection get]}
}

proc ListerSetup {this} {
	global scrlk mp3tool sends
	global configfile mp3tool sendlist global slist
	global tcl_platform

	set wait 1
	set scanned 0
	if [winfo exists .browse] {destroy .browse}
	if ![winfo exists .create] {
		toplevel .create
		wm title .create "MP3 Tool [Trans by] Cruise [Trans wait]!"
		label .create.t1 -width 30 -text [Trans please_wait]
		label .create.t2 -text [Trans loading_list_file]
		label .create.i -image wait$wait
		label .create.f -text "0 [Trans files_scanned]"
		pack .create.t1 -fill both -expand y
		pack .create.t2 -fill both -expand y
		pack .create.i -fill both -expand y
		pack .create.f -fill both -expand y
		update
	}
	if ![winfo exists .lmp3] {
		toplevel .lmp3
		wm title .lmp3 "MP3Tool [Trans list_browser]"
		wm geometry .lmp3 484x140
		set platform [array get tcl_platform platform]
		if {$platform == "platform unix"} {
			wm iconbitmap .lmp3 @$mp3tool(dir)/images/listviewer.xbm
		}
		frame .lmp3.l -relief sunken
		listbox .lmp3.l.list -borderwidth 2 -height 5 -width 1 \
			-yscrollcommand ".lmp3.l.scroll set" \
			-xscrollcommand ".lmp3.scrlx.scrollx set"
		frame .lmp3.scrlx
		scrollbar .lmp3.scrlx.scrollx -command ".lmp3.l.list xview" \
			-orient horizontal
		scrollbar .lmp3.l.scroll -command ".lmp3.l.list yview"
		frame .lmp3.ch -relief sunken -bg "light blue"
		frame .lmp3.ch.in -relief raised  -bg "light blue"
		frame .lmp3.ch.in.srch -bg "light blue"
		button .lmp3.ch.in.srch.sl -relief raised -text [Trans search_for] \
			-bg "light blue" -activeforeground red \
			-activebackground "light blue" \
			-command { SearchTXTList [.lmp3.ch.in.srch.entry get] }
		entry .lmp3.ch.in.srch.entry -relief sunken -width 20		
		label .lmp3.ch.in.srch.info -relief raised -borderwidth 2 -bg "light blue"
		pack .lmp3.ch -side top -fill x -expand n
		pack .lmp3.l -side bottom -fill both -expand y
		pack .lmp3.scrlx -side bottom -fill x -expand n
		pack .lmp3.scrlx.scrollx -side left -fill x -expand y
		pack .lmp3.ch.in -side left -expand y -fill both
		pack .lmp3.ch.in.srch -side bottom -expand y -fill both
		pack .lmp3.ch.in.srch.sl .lmp3.ch.in.srch.entry .lmp3.ch.in.srch.info -side left -fill both
		pack .lmp3.l.list -side left -fill both -expand y
		pack .lmp3.l.scroll -side top -fill y -expand y
		bind .lmp3.l.list <ButtonRelease> { ListSelectMP3 $mp3tool(chan) [selection get] }
		bind .lmp3.ch.in.srch.entry <Return> { SearchTXTList [.lmp3.ch.in.srch.entry get]}
		bind .lmp3.l.list <Return> { ListSelectMP3 $mp3tool(chan) [selection get] }
	}
	set infile [open $this r]
	gets $infile det
	set dettype 0
	set detstr "[Trans unknown_list_type]! ([Trans winging_it])"
	if {[string range $det 0 14] == "List generated " } {
		## MP3Tool List
		set detstr " MP3Tool [Trans list] ([Trans very_cool])"
		gets $infile
		gets $infile
		gets $infile
		gets $infile
		gets $infile
	} else {
		if {[string range $det 0 15] == "Current list of " } {
			## SDFind List (version 2.72)
			set detstr "SDFind [Trans list] ([Trans good])"
			gets $infile
		} else {
			if {[string range $det 0 13] == "**************"} {
				## SPR List (versions 3.8, 4.0, 4.91, 4.92 5.? 6.?)
				set detstr "SPR Jukebox [Trans list] (?[Trans error]?)"
				set dettype 1
				gets $infile wholine
				switch [string range $wholine 0 0] {
					< {
						set who [string range $wholine 9 [expr [string first "Multi-Media " $wholine] -4]]
					}
					L {
						 set who [string range $wholine 8 [expr [string first "MP3 " $wholine] -4]]
					}
				}
				gets $infile verline
				set starflag 0
				set spver [string range $verline  [string first "SpR JukeBox v" $verline]  [expr [string first "SpR JukeBox v" $verline] + 15]]
				switch -regexp -- $spver {
					v[2345].[0-9] {
						set detstr "SPR Jukebox [Trans list] (v2 -> v5)"
						set dettype 1
					}
					v6.[0123] {
						set detstr "SPR Jukebox [Trans list] (v6.1)"
						set dettype 1
					}
					v6.[456789] {
						set detstr "SPR Jukebox [Trans list] (v6.4 [Trans or_above])"
						set dettype 2
					}
					v[7-9].[1234567890] {
						set detstr "SPR Jukebox [Trans list] (v6.4 [Trans or_above])"
						set dettype 2
					}
				}
				switch $dettype {
					1 {
						while {[string range $det 0 5] != "-----\[" && $starflag < 2} {
							gets $infile det
							if {[eof $infile] == 1} {
								set detstr "[Trans error_in_list_detection]! (SpR)"
								set starflag 2
							}
						}
					}
					2 {
						while {$starflag < 2} {
							if {[string range $det 0 5] == "******"} {
								incr starflag
							}
							gets $infile det
							if {[eof $infile] == 1} {
								set detstr "[Trans error_in_list_detection]! (SpR)"
								set starflag 2
							}
						}
					}
				}
				gets $infile
			} else {
				if {[string range $det 0 3] == "<x> "} {
					## Gen X [Trans list] ([Trans version_unknown])
					set detstr "Gen X List ([Trans version_unknown])"
					while {[string range $det 0 5] != "------"} {
						gets $infile det
					}
				}
			}
		}
	}
	if {$mp3tool(totalfiles) <= 0} {CreateListQuestion 1}
	set count 0
	while {[eof $infile] != 1} {
		if [winfo exists .create] {
			incr wait
			incr scanned
			if {$wait > 19} {set wait 1}
			.create.i configure -image wait$wait
			.create.f configure -text "$scanned [Trans files_scanned]"
			if {$scanned % 100} {
				update idletasks
			} {
				update
			}
		} {
			toplevel .create
			wm title .create "MP3Tool [Trans by] Cruise [Trans wait]!"
			label .create.t1 -width 30 -text [Trans please_wait]
			label .create.t2 -text [Trans loading_list_file]
			label .create.i -image wait$wait
			label .create.f -text "0 [Trans files_scanned]"
			pack .create.t1 -fill both -expand y
			pack .create.t2 -fill both -expand y
			pack .create.i -fill both -expand y
			pack .create.f -fill both -expand y
			update
		}
		gets $infile input
		switch $dettype {
			0 {
				set brkpnt [string first ".mp3" [string tolower $input]]
				if {$brkpnt >= 4} {
					if {[HasListFile [string range $input 0 [expr $brkpnt +3]]] == "--"} {
						incr count
						.lmp3.l.list insert end [string range $input 0 [expr $brkpnt +3]]
					}
				}
			}
			1 {
				set fstr [string range $input [expr [string first "\]" $input] +2] [expr [string first "::" $input] -2]]
				set brkpnt [string first ".mp3" [string tolower $fstr]]
				if {$brkpnt >= 4} {
					if {[HasListFile [string range $fstr 0 [expr $brkpnt +3]]] == "--"} {
						incr count
						.lmp3.l.list insert end "!$who [string range $fstr 0 [expr $brkpnt +3]]"
					}
				}
			}
			2 {
				set fstr [string range $input [expr [string first " " $input] +1] [expr [string first "::" $input] -2]]
				set brkpnt [string first ".mp3" [string tolower $fstr]]
				if {$brkpnt >= 4} {
					if {[HasListFile [string range $fstr 0 [expr $brkpnt +3]]] == "--"} {
						incr count
						.lmp3.l.list insert end "!$who [string range $fstr 0 [expr $brkpnt +3]]"
					}
				}
			}
		}
	}
	close $infile	
	.lmp3.ch.in.srch.info configure -text "$detstr ($count [Trans files])"
	unset count
	if [winfo exists .create] {destroy .create}
}

proc WishList {} {
	global mp3tool tcl_platform

	set wait 1
	set scanned 0
	if ![winfo exists .create] {
		toplevel .create
		wm title .create "MP3 Tool [Trans by] Cruise [Trans wait]!"
		label .create.t1 -width 30 -text [Trans please_wait]
		label .create.t2 -text [Trans loading_list_file]
		label .create.i -image wait$wait
		label .create.f -text "0 [Trans files_scanned]"
		pack .create.t1 -fill both -expand y
		pack .create.t2 -fill both -expand y
		pack .create.i -fill both -expand y
		pack .create.f -fill both -expand y
		update
	}
	if ![winfo exists .wmp3] {
		toplevel .wmp3
		wm title .wmp3 "MP3Tool [Trans list_wish_browser]"
		set platform [array get tcl_platform platform]
		if {$platform == "platform unix"} {
			wm iconbitmap .wmp3 @$mp3tool(dir)/images/listviewer.xbm
		}
		frame .wmp3.l -relief sunken
		listbox .wmp3.l.list -borderwidth 2 -height 5 -width 65\
			-yscrollcommand ".wmp3.l.scrl.scroll set" \
			-xscrollcommand ".wmp3.scrlx.scrollx set"
		frame .wmp3.l.scrl
		frame .wmp3.scrlx
		scrollbar .wmp3.scrlx.scrollx -command ".wmp3.l.list xview" \
			-orient horizontal
		scrollbar .wmp3.l.scrl.scroll -command ".wmp3.l.list yview"
		frame .wmp3.ch -relief sunken -bg "light blue"
		frame .wmp3.ch.in -relief raised  -bg "light blue"
		frame .wmp3.ch.in.srch -bg "light blue"
		button .wmp3.ch.in.srch.sl -relief raised -text [Trans search_for] \
			-bg "light blue" -activeforeground red \
			-activebackground "light blue" \
			-command { SearchWISHList [.wmp3.ch.in.srch.entry get] }
		entry .wmp3.ch.in.srch.entry -relief sunken -width 20		
		label .wmp3.ch.in.srch.info -relief raised -borderwidth 2 -bg "light blue"
		pack .wmp3.ch -side top -fill both
		pack .wmp3.l -side bottom -fill both -expand y
		pack .wmp3.scrlx -side bottom -fill both
		pack .wmp3.scrlx.scrollx -side left -fill x -expand y
		pack .wmp3.ch.in -side left -expand y -fill both
		pack .wmp3.ch.in.srch -side bottom -expand y -fill both
		pack .wmp3.ch.in.srch.sl .wmp3.ch.in.srch.entry .wmp3.ch.in.srch.info -side left -fill both
		pack .wmp3.l.list .wmp3.l.scrl -side left -fill both -expand y
		pack .wmp3.l.scrl.scroll -side top -fill y -expand y
		bind .wmp3.l.list <ButtonRelease> { WishSelectMP3 $mp3tool(chan) [selection get] }
		bind .wmp3.ch.in.srch.entry <Return> { SearchWISHList [.wmp3.ch.in.srch.entry get]}
		bind .wmp3.l.list <Return> { WishSelectMP3 $mp3tool(chan) [selection get] }
	}
	if ![file exists $mp3tool(dir)/WishList.txt] {
		set outfile [open $mp3tool(dir)/WishList.txt w]
		close $outfile
	}
	set infile [open $mp3tool(dir)/WishList.txt r]
	set detstr "[Trans wish_list]! ([Trans yummy])"
	if {$mp3tool(totalfiles) == 0} {CreateListQuestion 1}
	set count 0
	while {[eof $infile] != 1} {
		if [winfo exists .create] {
			incr wait
			incr scanned
			if {$wait > 19} {set wait 1}
			.create.i configure -image wait$wait
			.create.f configure -text "$scanned [Trans files_scanned]"
			if {$scanned % 100} {
				update idletasks
			} {
				update
			}
		} {
			toplevel .create
			wm title .create "MP3Tool [Trans by] Cruise [Trans wait]!"
			label .create.t1 -width 30 -text [Trans please_wait]
			label .create.t2 -text [Trans loading_list_file]
			label .create.i -image wait$wait
			label .create.f -text "0 [Trans files_scanned]"
			pack .create.t1 -fill both -expand y
			pack .create.t2 -fill both -expand y
			pack .create.i -fill both -expand y
			pack .create.f -fill both -expand y
			update
		}
		gets $infile input
		set brkpnt [string first ".mp3" [string tolower $input]]
		if {$brkpnt >= 4} {
			if {[HasListFile [string range $input 0 [expr $brkpnt +3]]] == "--"} {
				incr count
				.wmp3.l.list insert end [string range $input 0 [expr $brkpnt +3]]
			}
		}
	}
	close $infile	
	.wmp3.ch.in.srch.info configure -text "$detstr ($count [Trans files])"
	unset count
	if [winfo exists .create] {destroy .create}
}

